@@ -9,13 +9,14 @@
 #include "log_pgsql_p.h"
 #include "log_pgsql.h"
 #include "messages.h"
+#include "crypto.h"
 
 #ifdef WITH_DMALLOC
 # include <dmalloc.h>
 #endif
 
 static size_t prv_PQescapeString(char * const to, const char * const from,
-                                 size_t length)
+                                 const size_t length)
 {
     register const char *source = from;
     register char *target = to;
@@ -26,14 +27,14 @@ static size_t prv_PQescapeString(char * const to, const char * const from,
         case 0:
             remaining = (size_t) 1U;
             break;
-        case '\n':
-            *target++ = '\\';
-            *target++ = 'n';
-            break;
         case '\r':
             *target++ = '\\';
             *target++ = 'r';            
             break;
+        case '\n':
+            *target++ = '\\';
+            *target++ = 'n';
+            break;
         case '\b':
             *target++ = '\\';
             *target++ = 'b';
@@ -190,7 +191,7 @@ static char *sqlsubst(const char *orig_str, char * const query,
                 *query_pnt++ = '\\';
             }
         }
-        if (((unsigned char) *orig_str_scan) < 32U) {
+        if (ISCTRLCODE(*orig_str_scan)) {
             goto nextone;
         }
         if (--query_len <= (size_t) 0U) {
@@ -251,21 +252,11 @@ static int pw_pgsql_connect(PGconn ** const id_sql_server)
     ret = 0;
     
     bye:
-    if (conninfo != NULL) {
-        free(conninfo);
-    }
-    if (escaped_server != NULL) {
-        free(escaped_server);
-    }
-    if (escaped_db != NULL) {
-        free(escaped_db);
-    }
-    if (escaped_user != NULL) {
-        free(escaped_user);
-    }
-    if (escaped_pw != NULL) {
-        free(escaped_pw);
-    }
+    free(conninfo);
+    free(escaped_server);
+    free(escaped_db);
+    free(escaped_user);
+    free(escaped_pw);
 
     return ret;
 }
@@ -296,7 +287,7 @@ static char *pw_pgsql_getquery(PGconn * const id_sql_server,
                                const char * const decimal_ip)
 {
     PGresult *qresult = NULL;
-    int length;
+    size_t length;
     char *answer = NULL;
     char query[PGSQL_MAX_REQUEST_LENGTH];
 
@@ -306,8 +297,9 @@ static char *pw_pgsql_getquery(PGconn * const id_sql_server,
     if (sqlsubst(orig_query, query, sizeof query,
                  account, ip, port, peer_ip, decimal_ip) == NULL) {
         goto bye;
-    }        
+    }
     if ((qresult = PQexec(id_sql_server, query)) == NULL) {
+        logfile(LOG_WARNING, MSG_SQL_WRONG_PARMS " : [%s]", query);        
         goto bye;
     }
     if (PQresultStatus(qresult) != PGRES_TUPLES_OK ||
@@ -316,12 +308,12 @@ static char *pw_pgsql_getquery(PGconn * const id_sql_server,
         PQgetisnull(qresult, 0, 0)) {
         goto bye;
     }
-    if ((length = PQgetlength(qresult, 0, 0)) <= 0 ||
-	(answer = malloc(length + 1U)) == NULL) {
+    if ((length = (size_t) PQgetlength(qresult, 0, 0) + (size_t) 1U)
+        <= (size_t) 1U || (answer = malloc(length)) == NULL) {
         goto bye;
     }
-    strncpy(answer, PQgetvalue(qresult, 0, 0), length); 
-    answer[length] = 0;
+    strncpy(answer, PQgetvalue(qresult, 0, 0), length - (size_t) 1U);
+    answer[length - (size_t) 1U] = 0;
     
     bye:
     if (qresult != NULL) {
@@ -338,8 +330,8 @@ void pw_pgsql_check(AuthResult * const result,
 {
     PGconn *id_sql_server = NULL;
     const char *spwd = NULL;           /* stored password */
-    const char *uid = NULL;            /* stored system login/uid */
-    const char *gid = NULL;            /* stored system group/gid */
+    const char *uid = sql_default_uid; /* stored system login/uid */
+    const char *gid = sql_default_gid; /* stored system group/gid */
     const char *dir = NULL;            /* stored home directory */
 #ifdef QUOTAS
     const char *sqta_fs = NULL;        /* stored quota files */    
@@ -360,7 +352,7 @@ void pw_pgsql_check(AuthResult * const result,
     char *escaped_decimal_ip = NULL;    
     char *scrambled_password = NULL;
     int committed = 1;
-    int crypto_crypt = 0, crypto_plain = 0;
+    int crypto_crypt = 0, crypto_plain = 0, crypto_md5 = 0;
     unsigned long decimal_ip_num = 0UL;
     char decimal_ip[42];
     char hbuf[NI_MAXHOST];
@@ -423,16 +415,22 @@ void pw_pgsql_check(AuthResult * const result,
                                   escaped_decimal_ip)) == NULL) {
         goto bye;
     }
-    if ((uid = pw_pgsql_getquery(id_sql_server, sqlreq_getuid,
-                                 escaped_account, escaped_ip, 
-                                 escaped_port, escaped_peer_ip,
-                                 escaped_decimal_ip)) == NULL) {
+    if (uid == NULL) {
+        uid = pw_pgsql_getquery(id_sql_server, sqlreq_getuid,
+                                escaped_account, escaped_ip, 
+                                escaped_port, escaped_peer_ip,
+                                escaped_decimal_ip);
+    }
+    if (uid == NULL) {
         goto bye;
     }
-    if ((gid = pw_pgsql_getquery(id_sql_server, sqlreq_getgid,
-                                 escaped_account, escaped_ip,
-                                 escaped_port, escaped_peer_ip,
-                                 escaped_decimal_ip)) == NULL) {
+    if (gid == NULL) {
+        gid = pw_pgsql_getquery(id_sql_server, sqlreq_getgid,
+                                escaped_account, escaped_ip,
+                                escaped_port, escaped_peer_ip,
+                                escaped_decimal_ip);
+    }
+    if (gid == NULL) {
         goto bye;
     }
     if ((dir = pw_pgsql_getquery(id_sql_server, sqlreq_getdir,
@@ -444,8 +442,11 @@ void pw_pgsql_check(AuthResult * const result,
     result->auth_ok--;                  /* -1 */
     if (strcasecmp(crypto, PASSWD_SQL_ANY) == 0) {
         crypto_crypt++;
+        crypto_md5++;
     } else if (strcasecmp(crypto, PASSWD_SQL_CRYPT) == 0) {
         crypto_crypt++;
+    } else if (strcasecmp(crypto, PASSWD_SQL_MD5) == 0) {
+        crypto_md5++;
     } else {                           /* default to plaintext */
         crypto_plain++;
     }
@@ -457,6 +458,14 @@ void pw_pgsql_check(AuthResult * const result,
             goto auth_ok;
         }
     }
+    if (crypto_md5 != 0) {
+        register const char *crypted;
+        
+        if ((crypted = (const char *) crypto_hash_md5(password, 1)) != NULL &&
+            strcmp(crypted, spwd) == 0) {
+            goto auth_ok;
+        }
+    }
     if (crypto_plain != 0) {
         if (*password != 0 &&    /* refuse null cleartext passwords */
             strcmp(password, spwd) == 0) {
@@ -658,7 +667,9 @@ void pw_pgsql_exit(void)
     ZFREE(crypto);
     ZFREE(sqlreq_getpw);
     ZFREE(sqlreq_getuid);
+    ZFREE(sql_default_uid);
     ZFREE(sqlreq_getgid);
+    ZFREE(sql_default_gid);
     ZFREE(sqlreq_getdir);
 #ifdef QUOTAS
     ZFREE(sqlreq_getqta_fs);
