@@ -90,24 +90,29 @@ const char *getname(const uid_t uid)
             return p->name;
         }
     }
-    if (chrooted == 0 && authresult.slow_tilde_expansion == 0) {
+    if (
+# ifndef ALWAYS_RESOLVE_IDS
+        chrooted == 0 && 
+# endif
+        authresult.slow_tilde_expansion == 0) {
         pwd = getpwuid(uid);
     }
     if ((p = malloc(sizeof *p)) == NULL) {
-        return "?       ";
+        die_mem();
     }
     p->uid = uid;
-    p->name = malloc(9);
-
-    if (p->name) {
-        if (pwd) {
-            if (SNCHECK(snprintf(p->name, 9, "%-8.8s", pwd->pw_name), 9)) {
-                _EXIT(EXIT_FAILURE);
-            }
-        } else {
-            if (SNCHECK(snprintf(p->name, 9, "%-8d", uid), 9)) {
-                _EXIT(EXIT_FAILURE);
-            }
+    if ((p->name = malloc((size_t) 9U)) == NULL) {
+        die_mem();
+    }
+    if (pwd != NULL) {
+        if (SNCHECK(snprintf(p->name, (size_t) 9U, 
+                             "%-8.8s", pwd->pw_name), (size_t) 9U)) {
+            _EXIT(EXIT_FAILURE);
+        }
+    } else {
+        if (SNCHECK(snprintf(p->name, (size_t) 9U, "%-8d", uid), 
+                    (size_t) 9U)) {
+            _EXIT(EXIT_FAILURE);
         }
     }
     p->next = user_head;
@@ -126,24 +131,29 @@ const char *getgroup(const gid_t gid)
         if (p->gid == gid) {
             return p->name;
         }
-    }    
-    if (chrooted == 0) {
+    } 
+# ifndef ALWAYS_RESOLVE_IDS   
+    if (chrooted == 0) 
+# endif
+    {
         pwd = getgrgid(gid);
     }
     if ((p = malloc(sizeof *p)) == NULL) {
-        return "?       ";        
+        die_mem();
     }
     p->gid = gid;
-    p->name = malloc(9);
-    if (p->name) {
-        if (pwd) {
-            if (SNCHECK(snprintf(p->name, 9, "%-8.8s", pwd->gr_name), 9)) {
-                _EXIT(EXIT_FAILURE);
-            }
-        } else {
-            if (SNCHECK(snprintf(p->name, 9, "%-8d", gid), 9)) {
-                _EXIT(EXIT_FAILURE);
-            }
+    if ((p->name = malloc((size_t) 9U)) == NULL) {
+        die_mem();
+    }
+    if (pwd != NULL) {
+        if (SNCHECK(snprintf(p->name, (size_t) 9U, "%-8.8s",
+                             pwd->gr_name), (size_t) 9U)) {
+            _EXIT(EXIT_FAILURE);
+        }
+    } else {
+        if (SNCHECK(snprintf(p->name, (size_t) 9U, "%-8d", gid), 
+                    (size_t) 9U)) {
+            _EXIT(EXIT_FAILURE);
         }
     }
     p->next = group_head;
@@ -156,7 +166,7 @@ const char *getgroup(const gid_t gid)
 static void addfile(const char *name, const char *suffix)
 {
     struct filename *p;
-    int l;
+    unsigned int l;
 
     if (!name || !suffix) {
         return;
@@ -165,18 +175,17 @@ static void addfile(const char *name, const char *suffix)
         return;
     }
     matches++;
-    l = (int) (strlen(name) + strlen(suffix));
+    l = (unsigned int) (strlen(name) + strlen(suffix));
     if (l > colwidth) {
         colwidth = l;
     }
-    /* Fixed off-by-one -J. */
     if ((p = malloc(offsetof(struct filename, line) + l + 1U)) == NULL) {
         return;
     }
     if (SNCHECK(snprintf(p->line, l + 1U, "%s%s", name, suffix), l + 1U)) {
         _EXIT(EXIT_FAILURE);
     }
-    if (tail) {
+    if (tail != NULL) {
         tail->down = p;
     } else {
         head = p;
@@ -237,9 +246,14 @@ static int listfile(const FileInfo * const fi,  const char *name)
         st.st_mode |= S_IFDIR;
     }  /* Hack to please some Windows client that dislike ../ -> ../ */
 #endif
-    t = localtime((time_t *) & st.st_mtime);
-    if (!t) {
-        die(421, LOG_ERR, "localtime()");
+#ifdef DISPLAY_FILES_IN_UTC_TIME
+    t = gmtime((time_t *) &st.st_mtime);
+#else
+    t = localtime((time_t *) &st.st_mtime);
+#endif
+    if (t == NULL) {
+        logfile(LOG_ERR, "{gm,local}gtime() for [%s]", name);
+        return 0;
     }
     if (opt_F) {
         if (S_ISLNK(st.st_mode))
@@ -252,8 +266,8 @@ static int listfile(const FileInfo * const fi,  const char *name)
         }
     }
     if (opt_l) {
-        strncpy(m, " ---------", (sizeof m) - 1U);
-        m[(sizeof m) - 1] = 0;
+        strncpy(m, " ---------", (sizeof m) - (size_t) 1U);
+        m[(sizeof m) - (size_t) 1U] = 0;
         switch (st.st_mode & S_IFMT) {
         case S_IFREG:
             m[0] = '-';
@@ -278,7 +292,7 @@ static int listfile(const FileInfo * const fi,  const char *name)
                 m[2] = 'w';
             }
             if (st.st_mode & 0100) {
-                m[3] = st.st_mode & 04000 ? 's' : 'x';
+                m[3] = (char) (st.st_mode & 04000 ? 's' : 'x');
             } else if (st.st_mode & 04000) {
                 m[3] = 'S';
             }
@@ -289,7 +303,7 @@ static int listfile(const FileInfo * const fi,  const char *name)
                 m[5] = 'w';
             }
             if (st.st_mode & 010) {
-                m[6] = st.st_mode & 02000 ? 's' : 'x';
+                m[6] = (char) (st.st_mode & 02000 ? 's' : 'x');
             } else if (st.st_mode & 02000) {
                 m[6] = 'S';
             }
@@ -300,7 +314,7 @@ static int listfile(const FileInfo * const fi,  const char *name)
                 m[8] = 'w';
             }
             if (st.st_mode & 01) {
-                m[9] = st.st_mode & 01000 ? 't' : 'x';
+                m[9] = (char) (st.st_mode & 01000 ? 't' : 'x');
             } else if (st.st_mode & 01000) {
                 m[9] = 'T';
             }            
@@ -364,7 +378,7 @@ static int listfile(const FileInfo * const fi,  const char *name)
                         suffix[0] = '*';
                     }
                 }
-                /* 2*MAXPATHLEN+gap should be enough, but be paranoid... */
+                /* 2 * MAXPATHLEN + gap should be enough, but be paranoid... */
                 if (SNCHECK
                     (snprintf(p, (sizeof_nameline) - strlen(alloca_nameline),
                               " -> %s", m), 
@@ -387,7 +401,7 @@ static int listfile(const FileInfo * const fi,  const char *name)
 
 static void outputfiles(int f)
 {
-    int n;
+    unsigned int n;
     struct filename *p;
     struct filename *q;
 
@@ -396,17 +410,17 @@ static void outputfiles(int f)
     }
     tail->down = NULL;
     tail = NULL;
-    colwidth = (colwidth | 7) + 1;
+    colwidth = (colwidth | 7U) + 1U;
     if (opt_l != 0 || opt_C == 0) {
-        colwidth = 75;
+        colwidth = 75U;
     }
     /* set up first column */
     p = head;
     p->top = 1;
-    if (colwidth > 75) {
+    if (colwidth > 75U) {
         n = filenames;
     } else {
-        n = (filenames + (75 / colwidth) - 1) / (75 / colwidth);
+        n = (filenames + (75U / colwidth) - 1U) / (75U / colwidth);
     }
     while (n && p) {
         p = p->down;
@@ -452,7 +466,7 @@ static void outputfiles(int f)
             if (q->right) {
                 memset(pad, '\t', sizeof pad - 1U);
                 pad[(sizeof pad) - 1] = 0;
-                pad[(colwidth + 7 - strlen(q->line)) / 8] = 0;
+                pad[(colwidth + 7U - strlen(q->line)) / 8] = 0;
             } else {
                 pad[0] = '\r';
                 pad[1] = '\n';
@@ -467,8 +481,8 @@ static void outputfiles(int f)
 
     /* reset variables for next time */
     head = tail = NULL;
-    colwidth = 0;
-    filenames = 0;
+    colwidth = 0U;
+    filenames = 0U;
 }
 
 /* functions to to sort for qsort() */
@@ -543,8 +557,8 @@ static FileInfo *sreaddir(char **names_pnt)
         free(names);
         return NULL;
     }
-    while ((de = readdir(d)) != NULL) {        
-        if (lstat(de->d_name, &st) < 0) {
+    while ((de = readdir(d)) != NULL) {
+        if (checkprintable(de->d_name) != 0 || lstat(de->d_name, &st) < 0) {
             continue;
         }
         name_len = strlen(de->d_name) + (size_t) 1U;        
@@ -615,7 +629,7 @@ static FileInfo *sreaddir(char **names_pnt)
     return files_info;
 }
 
-/* have to change to the directory first ( speed hack for -R ) */
+/* have to change to the directory first (speed hack for -R) */
 static void listdir(unsigned int depth, int f, const char *name)
 {
     FileInfo *dir;
@@ -700,67 +714,71 @@ static void listdir(unsigned int depth, int f, const char *name)
     names = NULL;
 }
 
-void donlist(char *arg, int on_ctrl_conn)
+void donlist(char *arg, const int on_ctrl_conn, const int opt_l_,
+             const int split_args)
 {
     int c;
 
     matches = 0U;
 
     opt_C = opt_d = opt_F = opt_R = opt_r = opt_t = opt_S = 0;
+    opt_l = opt_l_;
     if (force_ls_a != 0) {
         opt_a = 1;
     } else {
         opt_a = 0;
     }
-    while (isspace((unsigned char)*arg)) {
-        arg++;
-    }
-    while (arg && *arg == '-') {
-        while (arg++ && isalnum((unsigned char)*arg)) {
-            switch (*arg) {
-            case 'a':
-                opt_a = 1;
-                break;
-            case 'l':
-                opt_l = 1;
-                opt_C = 0;
-                break;
-            case '1':
-                opt_l = opt_C = 0;
-                break;
-            case 'C':
-                opt_l = 0;
-                opt_C = 1;
-                break;
-            case 'F':
-                opt_F = 1;
-                break;
-            case 'R':
-                opt_R = 1;
-                break;
-            case 'd':
-                opt_d = 1;
-                break;
-            case 'r':
-                opt_r = 1;
-                break;
-            case 't':
-                opt_t = 1;
-                opt_S = 0;
-                break;
-            case 'S':
-                opt_S = 1;
-                opt_t = 0;
-                break;
-            }
-        }
-        while (isspace((unsigned char)*arg)) {
+    if (split_args != 0) {
+        while (isspace((unsigned char) *arg)) {
             arg++;
         }
+        while (arg && *arg == '-') {
+            while (arg++ && isalnum((unsigned char) *arg)) {
+                switch (*arg) {
+                case 'a':
+                    opt_a = 1;
+                    break;
+                case 'l':
+                    opt_l = 1;
+                    opt_C = 0;
+                    break;
+                case '1':
+                    opt_l = opt_C = 0;
+                    break;
+                case 'C':
+                    opt_l = 0;
+                    opt_C = 1;
+                    break;
+                case 'F':
+                    opt_F = 1;
+                    break;
+                case 'R':
+                    opt_R = 1;
+                    break;
+                case 'd':
+                    opt_d = 1;
+                    break;
+                case 'r':
+                    opt_r = 1;
+                    break;
+                case 't':
+                    opt_t = 1;
+                    opt_S = 0;
+                    break;
+                case 'S':
+                    opt_S = 1;
+                    opt_t = 0;
+                    break;
+                }
+            }
+            while (isspace((unsigned char) *arg)) {
+                arg++;
+            }
+        }
     }
-
     if (on_ctrl_conn == 0) {
-        if ((c = opendata()) == -1) {
+	opendata();
+        if ((c = xferfd) == -1) {
             return;
         }
         doreply();
@@ -771,17 +789,19 @@ void donlist(char *arg, int on_ctrl_conn)
         c = 1;
         safe_write(c, "213-STAT" CRLF, sizeof "213-STAT" CRLF - 1U);
     }        
-    if (arg && *arg) {
+    if (arg != NULL && *arg != 0) {
         int justone;
 
         justone = 1;            /* just one argument, so don't print dir name */
 
-        while (arg) {
+        do {
             glob_t g;
             int a;
-            char *endarg = strchr(arg, ' ');
+            char *endarg;
 
-            if (endarg) {
+            if (split_args == 0) {
+                endarg = NULL;
+            } else if ((endarg = strchr(arg, ' ')) != NULL) {
                 *endarg++ = 0;
                 justone = 0;
             }
@@ -833,7 +853,7 @@ void donlist(char *arg, int on_ctrl_conn)
 #else
                             wrstr(c, "\r\n\r\n");
                             wrstr(c, *path);
-                            wrstr(c, ":\r\n\r\n");                                            
+                            wrstr(c, ":\r\n\r\n");
 #endif
                         }
                         if (!chdir(*path)) {
@@ -851,12 +871,12 @@ void donlist(char *arg, int on_ctrl_conn)
                     addreply(226, MSG_GLOB_READ_ERROR, arg);
                 } else if (a != GLOB_NOMATCH) {
                     addreply(226, MSG_GLOB_READ_ERROR, arg);
-                    addreply_noformat(0, MSG_PROBABLY_DENIED);                    
+                    addreply_noformat(0, MSG_PROBABLY_DENIED);
                 }
             }
             globfree(&g);
             arg = endarg;
-        }
+        } while (arg != NULL);
     } else {
         if (opt_d) {
             listfile(NULL, ".");
